@@ -7,6 +7,7 @@ from enum import Enum
 
 import pytest
 from qtpy.QtCore import Qt
+from qtpy.QtWidgets import QScrollArea
 
 from magicgui import magicgui, register_type, type_map, widgets
 from magicgui.signature import MagicSignature, magic_signature
@@ -50,7 +51,7 @@ def test_magicgui(magic_func):
     # we can delete widgets
     del magic_func.a
     with pytest.raises(AttributeError):
-        getattr(magic_func, "a")
+        magic_func.a
 
     # they disappear from the layout
     with pytest.raises(ValueError):
@@ -58,9 +59,9 @@ def test_magicgui(magic_func):
 
 
 def test_default_call_button_behavior(magic_func_defaults, magic_func_autocall):
-    assert magic_func_defaults._call_button is not None
+    assert magic_func_defaults.call_button is not None
 
-    assert magic_func_autocall._call_button is None
+    assert magic_func_autocall.call_button is None
     prior_autocall_count = magic_func_autocall.call_count
     magic_func_autocall.a.value = "hello"
     magic_func_autocall.b.value = 7
@@ -69,6 +70,7 @@ def test_default_call_button_behavior(magic_func_defaults, magic_func_autocall):
 
 def test_overriding_widget_type():
     """Test overriding the widget type of a parameter."""
+
     # a will now be a LineEdit instead of a spinbox
     @magicgui(a={"widget_type": "LineEdit"})
     def func(a: int = 1):
@@ -77,6 +79,13 @@ def test_overriding_widget_type():
     assert isinstance(func.a, widgets.LineEdit)
     assert func.a.value == "1"
 
+    # also without type annotation
+    @magicgui(a={"widget_type": "LogSlider"})
+    def g(a):
+        ...
+
+    assert isinstance(g.a, widgets.LogSlider)
+
 
 def test_unrecognized_types():
     """Test that arg with an unrecognized type is hidden."""
@@ -131,8 +140,8 @@ def test_call_button():
     def func(a: int, b: int = 3, c=7.1):
         assert a == 7
 
-    assert hasattr(func, "_call_button")
-    assert isinstance(func._call_button, widgets.PushButton)
+    assert hasattr(func, "call_button")
+    assert isinstance(func.call_button, widgets.PushButton)
     func.a.value = 7
 
 
@@ -358,7 +367,7 @@ def test_get_choices_raises():
     ids=["with-labels", "no-labels"],
 )
 def test_add_at_position(labels):
-    """Test that adding widghet with position option puts widget in the right place."""
+    """Test that adding widget with position option puts widget in the right place."""
 
     def func(a=1, b=2, c=3):
         pass
@@ -372,7 +381,7 @@ def test_add_at_position(labels):
 
     gui = magicgui(func, labels=labels)
     assert get_layout_items(gui) == ["a", "b", "c", "call_button"]
-    gui.insert(1, widgets.create_widget(name="new"))
+    gui.insert(1, widgets.create_widget(name="new", raise_on_unknown=False))
     assert get_layout_items(gui) == ["a", "new", "b", "c", "call_button"]
 
 
@@ -389,7 +398,7 @@ def test_show(magic_func):
     assert magic_func.visible
 
 
-def test_register_types():
+def test_register_types_by_string():
     """Test that we can register custom widget classes for certain types."""
     # must provide a non-None choices or widget_type
     with pytest.raises(ValueError):
@@ -425,8 +434,20 @@ def test_register_types():
     assert isinstance(func.c, widgets.ComboBox)
     assert isinstance(func.d, widgets.LineEdit)
 
-    del type_map._TYPE_DEFS[str]
-    del type_map._TYPE_DEFS[int]
+    del type_map._type_map._TYPE_DEFS[str]
+    del type_map._type_map._TYPE_DEFS[int]
+
+
+def test_register_types_by_class():
+    class MyLineEdit(widgets.LineEdit):
+        pass
+
+    class MyStr:
+        pass
+
+    register_type(MyStr, widget_type=MyLineEdit)
+    w = widgets.create_widget(value=MyStr())
+    assert isinstance(w, MyLineEdit)
 
 
 def test_register_return_callback():
@@ -444,19 +465,26 @@ def test_register_return_callback():
     register_type(int, return_callback=check_value)
     register_type(Base, return_callback=check_value)
 
-    @magicgui
-    def func(a=1) -> int:
-        return a
+    try:
 
-    func()
-    with pytest.raises(AssertionError):
-        func(3)
+        @magicgui
+        def func(a=1) -> int:
+            return a
 
-    @magicgui
-    def func2(a=1) -> Sub:
-        return a
+        func()
+        with pytest.raises(AssertionError):
+            func(3)
 
-    func2()
+        @magicgui
+        def func2(a=1) -> Sub:
+            return a
+
+        func2()
+    finally:
+        from magicgui.type_map._type_map import _RETURN_CALLBACKS
+
+        _RETURN_CALLBACKS.pop(int)
+        _RETURN_CALLBACKS.pop(Base)
 
 
 # @pytest.mark.skip(reason="need to rethink how to test this")
@@ -480,7 +508,7 @@ def test_function_binding():
     a = MyObject("a")
     b = MyObject("b")
 
-    assert a.method._call_button.text == "callme"  # type: ignore
+    assert a.method.call_button.text == "callme"  # type: ignore
     assert a.method.sigma.max == 365
     assert a.method() == ("a", 1)
     assert b.method(sigma=4) == ("b", 4)
@@ -488,6 +516,26 @@ def test_function_binding():
     assert b.method() == ("b", 5)
 
 
+def test_function_binding_multiple():
+    class MyObject:
+        def __init__(self):
+            pass
+
+        @magicgui
+        def method_0(self, sigma: float = 1):
+            pass
+
+        @magicgui
+        def method_1(self, sigma: float = 2):
+            pass
+
+    a = MyObject()
+    assert MyObject.method_0 is not a.method_0
+    assert a.method_0 is not a.method_1
+    assert a.method_0.sigma.value == 1
+    assert a.method_1.sigma.value == 2
+
+
 def test_call_count():
     """Test that a function gui remembers how many times it's been called."""
 
@@ -627,7 +675,8 @@ def test_no_tooltips_from_numpydoc():
 
 def test_only_some_tooltips_from_numpydoc():
     """Test that we can still show some tooltips with ``tooltips=False``."""
-    # tooltips=False, means docstrings wont be parsed at all, but tooltips
+
+    # tooltips=False, means docstrings won't be parsed at all, but tooltips
     # can still be manually provided.
     @magicgui(tooltips=False, y={"tooltip": "Still want a tooltip"})
     def func(x: int, y: str = "hi"):
@@ -646,7 +695,6 @@ def test_only_some_tooltips_from_numpydoc():
 
 
 def test_magicgui_type_error():
-
     with pytest.raises(TypeError):
         magicgui("not a function")  # type: ignore
 
@@ -705,7 +753,7 @@ def test_boolean_label():
 
 def test_none_defaults():
     """Make sure that an unannotated parameter with default=None is ok."""
-    assert widgets.create_widget(value=None).value is None
+    assert widgets.create_widget(value=None, raise_on_unknown=False).value is None
 
     def func(arg=None):
         return 1
@@ -785,3 +833,48 @@ def test_curry():
     assert isinstance(wdg.y, widgets.LineEdit)
     assert wdg2.y.value == "sdf"
     assert wdg2(1) == "sdf1"
+
+
+def test_scrollable():
+    @magicgui(scrollable=True)
+    def test_scrollable(a: int = 1, y: str = "a"):
+        ...
+
+    assert test_scrollable.native is not test_scrollable.root_native_widget
+    assert not isinstance(test_scrollable.native, QScrollArea)
+    assert isinstance(test_scrollable.root_native_widget, QScrollArea)
+
+    @magicgui(scrollable=False)
+    def test_nonscrollable(a: int = 1, y: str = "a"):
+        ...
+
+    assert test_nonscrollable.native is test_nonscrollable.root_native_widget
+    assert not isinstance(test_nonscrollable.native, QScrollArea)
+
+
+def test_unknown_exception_magicgui():
+    """Test that an unknown type is raised as a RuntimeError."""
+
+    class A:
+        pass
+
+    with pytest.raises(ValueError, match="No widget found for type"):
+
+        @magicgui(raise_on_unknown=True)
+        def func(a: A):
+            print(a)
+
+
+def test_unknown_exception_create_widget():
+    """Test that an unknown type is raised as a RuntimeError."""
+
+    class A:
+        pass
+
+    with pytest.raises(ValueError, match="No widget found for type"):
+        widgets.create_widget(A, raise_on_unknown=True)
+    with pytest.raises(ValueError, match="No widget found for type"):
+        widgets.create_widget(A)
+    assert isinstance(
+        widgets.create_widget(A, raise_on_unknown=False), widgets.EmptyWidget
+    )
